/*  VER 035   TAB P
 *  $Id: lock.c,v 1.6 1996/05/31 06:37:10 src Exp $
 *
 *  lock file handling 
 *
 *  assuming "spool" is the name of the outgoing spool,
 *  the following conventions apply:
 *
 *  for Cnews:
 *	general lock:	NEWSCTL/LOCKbatch
 *	spool file:	SPOOLDIR/out.going/spool/togo
 *	spool lock:	SPOOLDIR/out.going/spool/LOCKb
 *
 *  for Inn:
 *	general lock:	LOCKS/LOCK.sendbatch
 *	spool file:	SPOOLDIR/out.going/spool
 *	spool lock:	LOCKS/LOCK.spool
 *
 *	LOCKS is usually the same as NEWSCTL
 *	BUG: We are ignoring LOCK_STYLE
 *	BUG: What anout: shlock -p $$ -f lockfile
 *
 *  we don't need the general lock
 */

#include "postit.h"
#include "proto.h"

/*
 *  locking state
 */
static char name_temp[PATH_MAX];
static char name_lock[PATH_MAX];

static int lock_active = 0;
#define LOCK_TEMP 1
#define LOCK_LOCK 2

/*
 *  perform a lock
 */
void lock(
    char *dir_name, /* empty if in main lock */
    char *lock_name)
{
    long t = 0L;
    long locktime = timeout ? timeout : LOCKTIME;
    char pid[100];
    FILE *f;

    sprintf(pid,"%d",getpid());

    /* just in case */
    unlock();

    /* BUG: .. umask $NEWSUMASK */

    /* make the filenames */
    if (dir_name && dir_name[0]) {
	/* path specified */
	build_filename(name_lock,dir_name,"/",lock_name,NULL);
	build_filename(name_temp,name_lock,".",pid,NULL);
    } else {
	/* common lock directory */
	build_filename(name_lock,NEWSCTL,"/",lock_name,NULL);
	/* BUG: the Cnews convention is used for the temp_name */
	build_filename(name_temp,NEWSCTL,"/L.",pid,NULL);
    }

    /* build our own little file */
    if (!(f = fopen(name_temp,"w"))) {
	log_msg(L_ERRno,"cannot create lock '%s'", name_temp);
	unlock_exit(5);
    }
    lock_active |= LOCK_TEMP;
    fprintf(f,"%s\n",pid);
    if (fclose(f) == EOF) {
	log_msg(L_ERRno,"problems creating lock '%s'", name_temp);
	unlock_exit(5);
    }
    /* perform the locking action */
    for (;;) {
	if (link(name_temp,name_lock) >= 0) {
	    /* locked successfully */
	    lock_active |= LOCK_LOCK;
	    log_msg(L_DEBUG3,"created lock: %s",name_lock);
	    break;
	} 
	/* wait forever for access */
	if ((t += LOCKDELTA) > locktime) {
	    log_msg(L_ERR,"file lock '%s' timed out", name_lock);
	    unlock_exit(5);
	}
	sleep(LOCKDELTA);
    }
}

/*
 *  remove any locks
 */
void unlock(void)
{
    if (lock_active & LOCK_TEMP) {
	unlink(name_temp);
	lock_active &= ~LOCK_TEMP; 
    }
    if (lock_active & LOCK_LOCK) {
	unlink(name_lock);
	lock_active &= ~LOCK_LOCK; 
	log_msg(L_DEBUG3,"unlocked: %s", name_lock);
    }
}     

/*
 *  build a file name, max length PATH_MAX
 *  argument is NULL-terminated list 
 */
void build_filename(char *where, char *arg1, ...)      
{
    int len = 0;
    int n;
    char **argp;

    for (argp = &arg1; *argp; ++argp) {
	if ((len += (n=strlen(*argp))) >= PATH_MAX) {
	    log_msg(L_ERR,"file name longer than %d chars", PATH_MAX-1);
	    unlock_exit(1);
	}
	strcpy(where+len-n,*argp);
    }
}

/*
 *  perform exit(), but unlock() first
 */
void unlock_exit(int n)
{
    unlock();
    exit(n);
}

