.\" $Id: postit.man,v 1.7 1996/06/03 05:31:29 src Exp $
.SH NAME
.I postit
\- posts news articles to a remote NNTP server

.SH SYNOPSIS
.I postit
[
.B \-a
.I username/password
]
[
.B \-f
.I folder
]
[
.B \-v
]
[
.B \-r
]
[
.B \-i
]
[
.B \-d
]
[
.B \-n
]
[
.B \-p
]
[
.B \-t
.I timeout
]
[
.B \-l
.I logfile
]
[
.B \-s
.I spooldir
]
.I hostname
.I spoolname

.SH DESCRIPTION
.PP
.I Postit
is an NNTP client connecting to a
remote NNTP server and posting outgoing articles batched by the news system.
.PP
Together with	  
.IR "slurp" ","
it should provide the NNTP capabilities required for small news spools on
installations with NNTP access through limited personal
ISP accounts only.

.SH OPTIONS
.TP
.BI -a " username/password"
If specified, the username and password combination will be sent to
the server as part of the simple authorization protocol when the
connection is first made.
.TP
.BI -f " folder"
Successfully posted articles will be appended to the standard
mail folder indicated.
.TP
.B -r
A 'MODE READER' command is sent to the remote server. This is used when
connecting with a remote INN site which needs to be switched from innd to
nnrpd.
.TP
.B -i
Required if the local news spool is INN as opposed to Cnews. Controls
handling of spool and lock files.
.TP
.B -p
Keep the "Path". Normally, the "Path" is stripped together with
the "NNTP-Posting-Host" and "Xref" header items.
.TP
.BI -l " logfile"
Log information about successfully posted articles to the file indicated.
.TP
.I hostname
The hostname of the remote NNTP server to connect to. Must be
specified.
.TP
.I spoolname
The name of the outgoing spool queue.
If not specified, the hostname will be used.

.SH OPTIONS FOR TESTING
.TP
.B -d
Enable debugging. This diverts reporting to stderr instead of syslog
and turns on extra debugging output. Use two or three times 
for more output.
.TP
.B -n
No-action flag, will "fake" an NNTP connection, and leave the outgoing
batch untouched. Probably useful only with -d for debugging.
.TP
.B -o
Keep the previous spool in an ".old" file.
.TP
.BI -s " spooldir"
Use 
.I spooldir
as spool directory instead of the default
.BR "\*a" .
.TP
.BI -t " timeout"
Specify timeout for TCP/IP and lockfile operations, in seconds. The
default value is 600.
.TP
.B -v
Display version info. Include this when reporting bugs.

.SH OPERATION
.PP
The program should only be run by user
.IR news "."
When run,
.I postit
will go through the specified outgoing spool queue, and attempt to post all
articles therein to the NNTP server <hostname>. 
.PP
If a username and password for use with the simple authorization protocol
have been supplied, they will be sent to the server. If the
.B -r
option is specified, then a 'MODE READER' command will be sent.
.PP
When the transfer is finished, articles will be removed from the 
outgoing spool queue if successful transfer has been acknowledged by 
the NNTP host, if the article was specified by the host as a duplicate
(already posted), or if a spooled article can no longer be found.
.PP
Statistics on the connection will be logged to syslog (or stderr with
.BR -d .
The article count is the total number of
articles actually posted. The duplicate count is how many message IDs 
that were found to already exist on the remote system.
The missing count is those articles that could no longer be found
locally, usually because they were canceled locally, or perhaps since
they were expired already. If configured, the speed of transfer
of the article posting stage will also be logged.
.PP
A log file may be specified with the 
.B -l
option, and a folder of posted messages maintained with the
.B -f
option.

.SH LOG FILE FORMAT
.PP
The 
.I "-l"
log file will contain one line for each article. Each field is 
separated by a blank:
.IP "Month"
Standard 3 letter abbreviation. 
.IP "Date"
2 digits
.IP "HH:MM:SS"
Local time.			
.IP "Year"
4 digits.			
.IP "Spool"
Name of outgoing spool.        
.IP "<Msgid>"
Within angle brackets.	       
.IP "Filename"
As it appears in the spool file.
.IP "Sender"
The value of the Sender-field, or From-field if no Sender is available.
.IP "Status"
"OK" for successfully posted items, "Duplicate" for items already posted.
.IP "Lines"
Number of lines in the posting, separated from the status by a comma.

.SH DIAGNOSTICS
.PP
Postit returns error codes as follows:
.RS
0 - Successful completion
.br
1 - General system error
.br
2 - Incorrect arguments supplied.
.br
3 - Error connecting to remote host
.br
4 - NNTP Protocol error
.br
5 - Errors accessing spool file.
.RE

.SH BUGS
.PP
There should have been a maximum retry count on article posting,
depending on the nature of the error.
.PP
The folder will be updated regardless of whether the posting was
successfull or not, and the log file must be consulted.
.PP
Beta test version, various bugs might occur. The INN mode is 
an alpha test.

.SH FILES
.PP
For Cnews:
.IP "\*a/out.going/\fIspoolname\fP/togo"
Spool file 
.IP "\*a/out.going/\fIspoolname\fP/togo.old"
Previous spool file 
.IP "\*a/out.going/\fIspoolname\fP/LOCKb"
Lock file 
.PP
For INN:
.IP "\*a/out.going/\fIspoolname\fP"
Spool file
.IP "\*a/out.going/\fIspoolname\fP.old"
Previous spool file
.IP "\*c/LOCK.\fIspoolname\fP"
Lock file

.SH AUTHOR
Egil Kvaleberg <egilk@sn.no>

.SH "SEE ALSO"
.IR news (5),
.IR slurp (8)
.br
RFC977 \- Network News Transfer Protocol (NNTP)
.br
RFC1036 \- Usenet Article Format
